"use strict";

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _buildStoreChunk = require("../buildStoreChunk");

var _structure = require("../structure");

var _batchUpdates = require("../reducers/batchUpdates");

var _redux = require("redux");

var _isFunction = require("lodash/isFunction");

var _isFunction2 = _interopRequireDefault(_isFunction);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe("buildStoreChunk", function () {
  it("Will throw error if a structure is not defined", function () {
    expect(function () {
      return (0, _buildStoreChunk.buildStoreChunk)("toast");
    }).toThrowError(/structure/);
  });
  it("Will accept a single reducer (no nesting)", function () {
    expect(Object.keys((0, _buildStoreChunk.buildStoreChunk)("toast", _structure.Types.reducer(_structure.Types.string())))).toEqual(["reducers", "actions", "selectors"]);
  });
  it("Will return an object containing reducers, actions, and selectors as the result", function () {
    expect(Object.keys((0, _buildStoreChunk.buildStoreChunk)("toast", {
      example: _structure.Types.reducer(_structure.Types.string())
    }))).toEqual(["reducers", "actions", "selectors", "resetAll"]);
  });

  describe("Resulting chunk", function () {
    var chunk = (0, _buildStoreChunk.buildStoreChunk)("example", {
      nested1: _structure.Types.reducer(_structure.Types.string("foo")),
      nested2: _structure.Types.reducer(_structure.Types.shape({
        foo: _structure.Types.number(),
        bar: _structure.Types.string()
      })),
      nested3: _structure.Types.reducer(_structure.Types.arrayOf(_structure.Types.number(), [1, 2, 3])),
      nested4: _structure.Types.reducer({
        innerNested1: _structure.Types.reducer(_structure.Types.string("bar")),
        innerNested2: _structure.Types.reducer({
          innerNested3: _structure.Types.reducer(_structure.Types.string("baz"))
        })
      }),
      nested5: _structure.Types.reducer(_structure.Types.shape({
        arrayExample: _structure.Types.arrayOf(_structure.Types.string())
      }))
    });
    var nonNestedChunk = (0, _buildStoreChunk.buildStoreChunk)("example2", _structure.Types.reducer(_structure.Types.string("foo")));

    describe("Selectors", function () {
      var store = (0, _redux.createStore)((0, _redux.combineReducers)(_extends({}, chunk.reducers)));

      it("Selectors object has the correct top level structure for a nested chunk", function () {
        expect(Object.keys(chunk.selectors)).toEqual(["nested1", "nested2", "nested3", "nested4", "nested5"]);
      });
      it("Selectors object is a function for a non-nested chunk", function () {
        expect((0, _isFunction2.default)(nonNestedChunk.selectors)).toBe(true);
      });
      it("Nested selectors object has the correct structure for a defined reducer", function () {
        expect(Object.keys(chunk.selectors.nested4)).toEqual(["innerNested1", "innerNested2"]);
      });
      it("Selector returns correct value", function () {
        expect(chunk.selectors.nested1(store.getState())).toEqual("foo");
      });
      it("Nested selector returns correct value", function () {
        expect(chunk.selectors.nested4.innerNested1(store.getState())).toEqual("bar");
      });
    });

    describe("Actions", function () {
      it("Actions object has the correct top level structure for a nested chunk", function () {
        expect(Object.keys(chunk.actions)).toEqual(["nested1", "nested2", "nested3", "nested4", "nested5"]);
      });
      it("Actions object has the correct top level structure for a non nested chunk", function () {
        expect(Object.keys(nonNestedChunk.actions)).toEqual(["replace", "reset"]);
      });
      it("Nested actions object has the correct structure for a chunk", function () {
        expect(Object.keys(chunk.actions.nested4)).toEqual(["innerNested1", "innerNested2"]);
      });
      it("Replace actions return an object that contains a type and payload", function () {
        expect(Object.keys(chunk.actions.nested1.replace("bar"))).toEqual(["type", "payload"]);
        expect(Object.keys(chunk.actions.nested2.replace({}))).toEqual(["type", "payload"]);
        expect(Object.keys(chunk.actions.nested3.replace([]))).toEqual(["type", "payload", "index"]);
      });
    });

    describe("Combined actions and selectors (nested chunk)", function () {
      var store = (0, _redux.createStore)((0, _redux.combineReducers)(_extends({}, chunk.reducers, nonNestedChunk.reducers)));

      it("Dispatching an action should correctly update the store", function () {
        store.dispatch(chunk.actions.nested1.replace("bar"));
        expect(chunk.selectors.nested1(store.getState())).toEqual("bar");

        store.dispatch(chunk.actions.nested1.reset());
        expect(chunk.selectors.nested1(store.getState())).toEqual("foo");
      });

      it("Dispatching an empty array property should replace existing array", function () {
        store.dispatch(chunk.actions.nested5.replace({ arrayExample: ["2"] }));
        store.dispatch(chunk.actions.nested5.update({ arrayExample: [] }));
        expect(chunk.selectors.nested5(store.getState())).toEqual({
          arrayExample: []
        });
      });
    });

    describe("Combined actions and selectors (non nested chunk)", function () {
      var store = (0, _redux.createStore)((0, _redux.combineReducers)(_extends({}, chunk.reducers, nonNestedChunk.reducers)));

      it("Dispatching an action should correctly update the store", function () {
        store.dispatch(nonNestedChunk.actions.replace("bar"));
        expect(nonNestedChunk.selectors(store.getState())).toEqual("bar");

        store.dispatch(nonNestedChunk.actions.reset());
        expect(nonNestedChunk.selectors(store.getState())).toEqual("foo");
      });
    });

    describe("Combined actions", function () {
      var store = (0, _redux.createStore)((0, _redux.combineReducers)(_extends({}, chunk.reducers, nonNestedChunk.reducers)));

      it("Dispatching a createCombinedAction updates the store correctly", function () {
        store.dispatch((0, _batchUpdates.createCombinedAction)({
          name: "batchUpdateFunsies",
          actions: [nonNestedChunk.actions.replace("bar"), chunk.actions.nested2.update({
            foo: 4
          }), chunk.actions.nested2.update({
            bar: "boop!"
          }), chunk.actions.nested3.replace([4, 5, 6]), chunk.actions.nested3.removeAtIndex(1)]
        }));
        expect(nonNestedChunk.selectors(store.getState())).toEqual("bar");
        expect(chunk.selectors.nested2(store.getState())).toEqual({
          foo: 4,
          bar: "boop!"
        });
        expect(chunk.selectors.nested3(store.getState())).toEqual([4, 6]);
      });

      describe("reset all action", function () {
        it("Calling reset all will reset all store chunks", function () {
          store.dispatch((0, _batchUpdates.createCombinedAction)({
            name: "batchUpdateFunsies",
            actions: [chunk.actions.nested2.update({
              foo: 4
            }), chunk.actions.nested2.update({
              bar: "boop!"
            }), chunk.actions.nested3.replace([4, 5, 6]), chunk.actions.nested3.removeAtIndex(1), chunk.actions.nested4.innerNested1.replace("boop!")]
          }));

          store.dispatch(chunk.resetAll());
          expect(chunk.selectors.nested2(store.getState())).toEqual({
            foo: 0,
            bar: ""
          });
          expect(chunk.selectors.nested3(store.getState())).toEqual([1, 2, 3]);
          expect(chunk.selectors.nested4.innerNested1(store.getState())).toBe("bar");
        });
      });
    });
  });
});