"use strict";

var _structure = require("../structure");

var _validatePayload = require("../validatePayload");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe("Validation functionality", function () {
  describe("Primitives/Custom", function () {
    var customType = _structure.Types.custom({
      validator: function validator(value) {
        return value === 3;
      },
      validationErrorMessage: function validationErrorMessage(value) {
        return "Oh noes! " + value;
      }
    })();

    it("Number primitive should allow for numbers", function () {
      expect((0, _validatePayload.validateValue)(_structure.Types.number(), 3)).toBe(3);
    });
    it("String primitive should allow for string", function () {
      expect((0, _validatePayload.validateValue)(_structure.Types.string(), "toast")).toBe("toast");
    });
    it("Boolean primitive should allow for string", function () {
      expect((0, _validatePayload.validateValue)(_structure.Types.boolean(), true)).toBe(true);
    });
    it("Any should allow for anything", function () {
      var date = new Date();
      expect((0, _validatePayload.validateValue)(_structure.Types.any(), date)).toEqual(date);
    });
    it("should validate custom values using the custom validator", function () {
      expect((0, _validatePayload.validateValue)(customType, 3)).toBe(3);
    });
    it("should return undefined from custom validators which failed", function () {
      expect((0, _validatePayload.validateValue)(customType, 4)).toBeUndefined();
    });
  });

  describe("Arrays", function () {
    var testArrayStructure = _structure.Types.arrayOf(_structure.Types.string());
    it("Arrays should allow for primitives", function () {
      expect((0, _validatePayload.validateArray)(testArrayStructure, ["a", "b", "c", "d"])).toEqual(["a", "b", "c", "d"]);
    });
    it("Arrays should strip values for primitives which fail the test", function () {
      expect((0, _validatePayload.validateArray)(testArrayStructure, ["a", "b", 3, "d"])).toEqual(["a", "b", "d"]);
    });

    var testArrayStructure2 = _structure.Types.arrayOf(_structure.Types.shape({
      test1: _structure.Types.number()
    }));
    it("Arrays should allow for complex objects", function () {
      expect((0, _validatePayload.validateArray)(testArrayStructure2, [{ test1: 3 }, { test1: 4 }])).toEqual([{ test1: 3 }, { test1: 4 }]);
    });
    var testArrayStructure3 = _structure.Types.arrayOf(_structure.Types.shape({
      test1: _structure.Types.arrayOf(_structure.Types.number()),
      test2: _structure.Types.custom({ validator: function validator(value) {
          return value === "foo";
        } })()
    }));
    it("Arrays should allow for complex objects - test 2", function () {
      expect((0, _validatePayload.validateArray)(testArrayStructure3, [{ test1: [3, 4, 5], test2: "foo" }])).toEqual([{ test1: [3, 4, 5], test2: "foo" }]);
    });
    it("Array should return an empty array if a non-array is passed", function () {
      expect((0, _validatePayload.validateArray)("foo")).toEqual([]);
    });
    it("Array should allow an empty array", function () {
      expect((0, _validatePayload.validateArray)([])).toEqual([]);
    });
  });

  describe("Objects", function () {
    var testObjectStructure = _structure.Types.shape({
      test1: _structure.Types.string(),
      test2: _structure.Types.number(),
      test3: _structure.Types.custom({ validator: function validator(value) {
          return value !== 4;
        } })()
    });
    it("Object of primitives should allow all props present in the structure", function () {
      expect((0, _validatePayload.validateShape)(testObjectStructure, {
        test1: "toast",
        test2: 3,
        test3: 1
      })).toEqual({ test1: "toast", test2: 3, test3: 1 });
    });
    it("Object of primitives should only allow for props with values which match their config", function () {
      expect((0, _validatePayload.validateShape)(testObjectStructure, { test1: 5, test2: 3, test3: 4 })).toEqual({ test2: 3 });
    });
    it("Object of primitives should strip any properties not part of the config", function () {
      expect((0, _validatePayload.validateShape)(testObjectStructure, {
        test1: "toast",
        test2: 3,
        toast: "bar"
      })).toEqual({ test1: "toast", test2: 3 });
    });

    var testObjectStructure2 = _structure.Types.shape({
      test1: testObjectStructure
    });
    it("Objects should allow for arbitrary nesting of objects", function () {
      expect((0, _validatePayload.validateShape)(testObjectStructure2, {
        test1: { test1: "toast", test2: 3 }
      })).toEqual({ test1: { test1: "toast", test2: 3 } });
    });

    var testObjectStructure3 = _structure.Types.shape({
      test1: _structure.Types.shape({
        test2: _structure.Types.string()
      }),
      test2: _structure.Types.string()
    });
    it("Objects containing objects should properly check if an object is provided", function () {
      expect((0, _validatePayload.validateShape)(testObjectStructure3, { test1: "foo", test2: "bar" })).toEqual({
        test1: {},
        test2: "bar"
      });
    });

    var testObjectStructure4 = _structure.Types.shape(_defineProperty({
      test2: _structure.Types.string()
    }, _structure.Types.wildcardKey(), _structure.Types.string()));
    var testObjectStructure5 = _structure.Types.shape(_defineProperty({
      test2: _structure.Types.string()
    }, _structure.Types.wildcardKey(), _structure.Types.any()));
    it("Should, if a key is not specified, see if the key matches the wildcard type, and apply if true", function () {
      expect((0, _validatePayload.validateShape)(testObjectStructure4, { test1: "foo", test2: "bar" })).toEqual({
        test1: "foo",
        test2: "bar"
      });

      expect((0, _validatePayload.validateShape)(testObjectStructure5, { test1: 0, test2: "bar" })).toEqual({
        test1: 0,
        test2: "bar"
      });
    });

    var testObjectStructure6 = _structure.Types.shape(_defineProperty({
      test2: _structure.Types.string()
    }, _structure.Types.wildcardKey(), _structure.Types.string()));
    it("Should, if a key is not specified, and does not match the wildcardKey, strip it out", function () {
      expect((0, _validatePayload.validateShape)(testObjectStructure6, { test1: 0, test2: "bar" })).toEqual({
        test2: "bar"
      });
    });

    var testObjectStructure7 = _structure.Types.shape({
      test1: _structure.Types.arrayOf(_structure.Types.string())
    });
    it("Should allow an empty array to be passed for an array property", function () {
      expect((0, _validatePayload.validateShape)(testObjectStructure7, { test1: [] })).toEqual({
        test1: []
      });
    });
  });

  describe("Non covered types", function () {
    it("A type with no associated validation should throw an error", function () {
      expect(function () {
        return (0, _validatePayload.getTypeValidation)("toast");
      }).toThrowError(/validation/);
    });
  });

  describe("Has wildcard value", function () {
    var testObjectStructure = _structure.Types.shape(_defineProperty({
      test1: _structure.Types.string(),
      test2: _structure.Types.number()
    }, _structure.Types.wildcardKey(), _structure.Types.any()));

    var testObjectStructure2 = _structure.Types.shape({
      test1: _structure.Types.string(),
      test2: _structure.Types.number()
    });

    it("should return true if the objectStructure passed in has a wildcard key", function () {
      expect((0, _validatePayload.hasWildcardKey)(testObjectStructure)).toBe(true);
    });

    it("should return false if no wildcard key passed in", function () {
      expect((0, _validatePayload.hasWildcardKey)(testObjectStructure2)).toBe(false);
    });
  });

  describe("GetValueType", function () {
    var testObjectStructure = _structure.Types.shape(_defineProperty({
      test1: _structure.Types.string(),
      test2: _structure.Types.number()
    }, _structure.Types.wildcardKey(), _structure.Types.number()));

    var testObjectStructure2 = _structure.Types.shape({
      test1: _structure.Types.string(),
      test2: _structure.Types.number()
    });

    it("should return the correct type for a key that is present, if no wildcard present", function () {
      expect((0, _validatePayload.getValueType)(testObjectStructure, "test1", false)().type).toEqual(_structure.Types.string()().type);
    });

    it("should return the wildcard value if key not present and wildcard is", function () {
      expect((0, _validatePayload.getValueType)(testObjectStructure, "test3", true)().type).toEqual(_structure.Types.number()().type);
    });

    it("should return undefined if no wildcard or matching key", function () {
      expect((0, _validatePayload.getValueType)(testObjectStructure, "test3", false)).toEqual(undefined);
    });
  });
});