'use strict';

var _arrayReducer = require('../arrayReducer');

var _reducers = require('../../reducers');

var _structure = require('../../structure');

describe('arrayReducer', function () {

    describe('behaviors', function () {

        describe('replaceAtIndex', function () {
            var replaceAtIndex = _arrayReducer.DEFAULT_ARRAY_BEHAVIORS.replaceAtIndex;

            it('should update at index correctly', function () {
                expect(replaceAtIndex.reducer([1, 2, 3], 4, [], 0)).toEqual([4, 2, 3]);
            });
            it('should return state if index not passed', function () {
                expect(replaceAtIndex.reducer([1, 2, 3], 4, [])).toEqual([1, 2, 3]);
            });
            it('should return state if no payload passed', function () {
                expect(replaceAtIndex.reducer([1, 2, 3], undefined, [], 0)).toEqual([1, 2, 3]);
            });
        });

        describe('resetAtIndex', function () {
            var resetAtIndex = _arrayReducer.DEFAULT_ARRAY_BEHAVIORS.resetAtIndex;

            it('should reset at index correctly', function () {
                expect(resetAtIndex.reducer([1, 2, 3], undefined, 0, 0)).toEqual([0, 2, 3]);
            });
            it('should return state if no index provided', function () {
                expect(resetAtIndex.reducer([1, 2, 3], undefined, 0)).toEqual([1, 2, 3]);
            });
        });

        describe('removeAtIndex', function () {
            var removeAtIndex = _arrayReducer.DEFAULT_ARRAY_BEHAVIORS.removeAtIndex;

            it('should remove at index correctly', function () {
                expect(removeAtIndex.reducer([1, 2, 3], 0)).toEqual([2, 3]);
            });
            it('should return state if no index provided', function () {
                expect(removeAtIndex.reducer([1, 2, 3])).toEqual([1, 2, 3]);
            });
        });

        describe('replace', function () {
            var replace = _arrayReducer.DEFAULT_ARRAY_BEHAVIORS.replace;

            it('should return state if payload is not an array', function () {
                expect(replace.reducer([1, 2, 3], '')).toEqual([1, 2, 3]);
            });
            it('should return the new array', function () {
                expect(replace.reducer([1, 2, 3], [4, 5, 6])).toEqual([4, 5, 6]);
            });
        });

        describe('reset', function () {
            var reset = _arrayReducer.DEFAULT_ARRAY_BEHAVIORS.reset;

            it('should reset the state', function () {
                expect(reset.reducer([1, 2, 3], undefined, [4, 5, 6])).toEqual([4, 5, 6]);
            });
        });

        describe('push', function () {
            var push = _arrayReducer.DEFAULT_ARRAY_BEHAVIORS.push;

            it('should push the payload onto the end of the array', function () {
                expect(push.reducer([1, 2, 3], 4)).toEqual([1, 2, 3, 4]);
            });
        });

        describe('pushOrRemove', function () {
            var pushOrRemove = _arrayReducer.DEFAULT_ARRAY_BEHAVIORS.pushOrRemove;

            it('should push the primitive payload onto the end of the array', function () {
                expect(pushOrRemove.reducer([1, 2, 3], 4)).toEqual([1, 2, 3, 4]);
            });
            it('should remove the primitive payload from the array', function () {
                expect(pushOrRemove.reducer([1, 2, 3], 2)).toEqual([1, 3]);
            });
            it('should push the object payload onto the end of the array', function () {
                var users = [{ 'user': 'barney', 'active': false }, { 'user': 'fred', 'active': false }];
                var expected_users = [{ 'user': 'barney', 'active': false }, { 'user': 'fred', 'active': false }, { 'user': 'pebbles', 'active': true }];
                expect(pushOrRemove.reducer(users, { 'user': 'pebbles', 'active': true })).toEqual(expected_users);
            });
            it('should remove the object payload from the array', function () {
                var users = [{ 'user': 'barney', 'active': false }, { 'user': 'fred', 'active': false }, { 'user': 'pebbles', 'active': true }];
                var expected_users = [{ 'user': 'barney', 'active': false }, { 'user': 'pebbles', 'active': true }];
                expect(pushOrRemove.reducer(users, { 'user': 'fred', 'active': false })).toEqual(expected_users);
            });
        });

        describe('pop', function () {
            var pop = _arrayReducer.DEFAULT_ARRAY_BEHAVIORS.pop;

            it('should remove the last element from the array', function () {
                expect(pop.reducer([1, 2, 3])).toEqual([1, 2]);
            });
        });

        describe('unshift', function () {
            var unshift = _arrayReducer.DEFAULT_ARRAY_BEHAVIORS.unshift;

            it('should add the payload to the beginning of the array', function () {
                expect(unshift.reducer([1, 2, 3], 4)).toEqual([4, 1, 2, 3]);
            });
        });

        describe('shift', function () {
            var shift = _arrayReducer.DEFAULT_ARRAY_BEHAVIORS.shift;

            it('should remove the first element of the array', function () {
                expect(shift.reducer([1, 2, 3])).toEqual([2, 3]);
            });
        });
    });

    describe('applyValidation', function () {
        var arrayStructure = _structure.Types.arrayOf(_structure.Types.number());
        var arrayStructure2 = _structure.Types.arrayOf(_structure.Types.shape({ foo: _structure.Types.string() }));
        it('should validate arrays correctly', function () {
            expect((0, _arrayReducer.applyValidation)(arrayStructure, [1, 2, 3])).toEqual([1, 2, 3]);
            expect((0, _arrayReducer.applyValidation)(arrayStructure, [1, 'foo', 3])).toEqual([1, 3]);
        });
        it('should validate non array primitive payloads correctly', function () {
            expect((0, _arrayReducer.applyValidation)(arrayStructure, 1)).toEqual(1);
        });
        it('should validate none array object payloads correctly', function () {
            expect((0, _arrayReducer.applyValidation)(arrayStructure2, { foo: 'toast' })).toEqual({ foo: 'toast' });
        });
    });

    describe('createReducer', function () {
        var arrayStructure = _structure.Types.arrayOf(_structure.Types.number(), [1, 2, 3, 4]);
        var reducer = (0, _arrayReducer.createReducer)(arrayStructure, (0, _reducers.createReducerBehaviors)(_arrayReducer.DEFAULT_ARRAY_BEHAVIORS, 'string'));

        it('should call the correct behavior', function () {
            expect(reducer([1, 2, 3], {
                type: 'string.replaceAtIndex',
                payload: 4,
                index: 0
            })).toEqual([4, 2, 3]);
        });

        it('do not trigger validation if not required', function () {
            expect(reducer([1, 2], {
                type: 'string.reset'
            })).toEqual([1, 2, 3, 4]);
        });
    });
});