'use strict';

var _objectReducer = require('../objectReducer');

var _reducers = require('../../reducers');

var _structure = require('../../structure');

describe('ObjectReducer', function () {

    describe('behaviors', function () {

        describe('replace', function () {
            var replace = _objectReducer.DEFAULT_SHAPE_BEHAVIORS.replace;

            it('reducer should return the state if the payload is undefined', function () {
                expect(replace.reducer({ foo: 1 }, undefined)).toEqual({ foo: 1 });
            });
            it('reducer should return the new state combined with the initial state', function () {
                expect(replace.reducer({ foo: 1, bar: 3 }, { foo: 2 }, { foo: 1, bar: 5 })).toEqual({ foo: 2, bar: 5 });
            });
        });

        describe('reset', function () {
            var reset = _objectReducer.DEFAULT_SHAPE_BEHAVIORS.reset;

            it('reducer should return the initial state', function () {
                expect(reset.reducer({ foo: 1 }, undefined, { foo: 2 })).toEqual({ foo: 2 });
            });

            it('reducer should apply the payload (if an object) to the store over the default state', function () {
                expect(reset.reducer({ foo: 23, bar: 4 }, { foo: 1 }, { foo: 4, bar: 5 })).toEqual({ foo: 1, bar: 5 });
            });

            it('reducer should return initial state if the payload is not an object', function () {
                expect(reset.reducer({ foo: 23, bar: 4 }, 'nope!', { foo: 4, bar: 5 })).toEqual({ foo: 4, bar: 5 });
            });
        });

        describe('update', function () {
            var update = _objectReducer.DEFAULT_SHAPE_BEHAVIORS.update;

            it('reducer should return the shallow merged new state', function () {
                expect(update.reducer({ foo: 1, bar: 2 }, { foo: 3 })).toEqual({ foo: 3, bar: 2 });
            });
            it('reducer should return state if payload is not an object', function () {
                expect(update.reducer({ foo: 1, bar: 2 }, 'toast')).toEqual({ foo: 1, bar: 2 });
            });
        });
    });

    describe('calculateDefaults', function () {
        var structure = {
            foo: _structure.Types.string('foo'),
            bar: _structure.Types.shape({
                baz: _structure.Types.number(5)
            })
        };

        expect((0, _objectReducer.calculateDefaults)(structure)).toEqual({
            foo: 'foo',
            bar: {
                baz: 5
            }
        });
    });

    describe('createReducer', function () {
        var shapeStructure = _structure.Types.shape({
            foo: _structure.Types.string('foo'),
            bar: _structure.Types.shape({
                baz: _structure.Types.number(5)
            })
        });
        var reducer = (0, _objectReducer.createReducer)(shapeStructure, (0, _reducers.createReducerBehaviors)(_objectReducer.DEFAULT_SHAPE_BEHAVIORS, 'string'));
        it('call the correct behavior', function () {
            expect(reducer({ foo: 'foo', bar: { baz: 5 } }, {
                type: 'string.update',
                payload: { foo: 'toast' }
            })).toEqual({ foo: 'toast', bar: { baz: 5 } });
        });

        it('do not trigger validation if not required', function () {
            expect(reducer({ foo: 'foo', bar: { baz: 6 } }, {
                type: 'string.reset'
            })).toEqual({ foo: 'foo', bar: { baz: 5 } });
        });
    });
});